@@ -1,3 +1,30 @@
+# Copyright (C) Tal Galili
+#
+# This file is part of installr.
+#
+# installr is free software: you can redistribute it and/or modify it
+# under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 2 of the License, or
+# (at your option) any later version.
+#
+# installr is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+#
+
+
+
+
+
+
+
+
+
+
 #' @title Check and Create MD5 Checksum Files
 #' @export
 #' @description checkMD5sums checks the files against a file 'MD5'.  This extends the default checkMD5sums from package tools by adding a new parameter "md5file"
@@ -230,6 +257,7 @@ browse.latest.R.NEWS <- function(
 #' @param to_checkMD5sums Should we check that the new R installation has the files we expect it to (by checking the MD5 sums)? default is TRUE.  It assumes that the R which was isntalled is the latest R version.
 #' @param keep_install_file If TRUE - the installer file will not be erased after it is downloaded and run.
 #' @param download_dir A character of the directory into which to download the file. (default is \link{tempdir}())
+#' @param silent If TRUE - enables silent installation mode.
 #' @param ... extra parameters to pass to \link{install.URL}
 #' @return TRUE/FALSE - was the installation of R successful or not.
 #' @seealso \link{uninstall.R}, \link{install.Rdevel}, \link{updateR}, \link{system}
@@ -243,6 +271,7 @@ install.R <- function(page_with_download_url = "http://cran.rstudio.com/bin/wind
                       to_checkMD5sums = TRUE,
                       keep_install_file = FALSE,
                       download_dir = tempdir(),
+                      silent = FALSE,
                       ...) {
    # I'm using the rsudio cran since it redirects to other servers wourld wide.
    # here there is a question on how to do it with the different mirrors. (maybe to add it as an option?)
@@ -254,7 +283,14 @@ install.R <- function(page_with_download_url = "http://cran.rstudio.com/bin/wind
    exe_filename   <- regmatches(target_line, m) 
    URL <- paste(page_with_download_url, exe_filename, sep = '')
    
-   did_R_install <- install.URL(URL, keep_install_file = keep_install_file,download_dir=download_dir, ...)
+   # add command line arguments for silent mode
+   if(silent){
+      installer_option <- "/SILENT"
+   } else{
+      installer_option <- NULL
+   }
+   
+   did_R_install <- install.URL(URL, keep_install_file = keep_install_file,download_dir=download_dir, installer_option = installer_option, ...)
    if(!did_R_install) return(FALSE) 
    
    # checks the MD5sums from the new R installation:
@@ -617,6 +653,7 @@ copy.packages.between.libraries <- function(from, to, ask =FALSE,keep_old = TRUE
 #' @param to_checkMD5sums Should we check that the new R installation has the files we expect it to (by checking the MD5 sums)? default is TRUE.  It assumes that the R which was isntalled is the latest R version. parameter is passed to install.R()
 #' @param keep_install_file If TRUE - the installer file will not be erased after it is downloaded and run.
 #' @param download_dir A character of the directory into which to download the file. (default is \link{tempdir}())
+#' @param silent If TRUE - enables silent installation mode.
 #' @param ... Other arguments (this is currently not used in any way)
 #' @return a TRUE/FALSE value on whether or not R was updated.
 #' @seealso \link{check.for.updates.R}, \link{install.R}, 
@@ -633,7 +670,7 @@ copy.packages.between.libraries <- function(from, to, ask =FALSE,keep_old = TRUE
 #' updateR() # will ask you what you want at every decision.
 #' }
 updateR <- function(browse_news, install_R, copy_packages, keep_old_packages,  update_packages, start_new_R, quit_R,  print_R_versions=TRUE, use_GUI = TRUE, 
-                    to_checkMD5sums = TRUE, keep_install_file = FALSE, download_dir = tempdir(), ...) {
+                    to_checkMD5sums = TRUE, keep_install_file = FALSE, download_dir = tempdir(), silent = FALSE, ...) {
    # this function checks if we have the latest version of R
    # IF not - it notifies the user - and leaves.
    # If there is a new version - it offers the user to download and install it.   
@@ -658,7 +695,7 @@ updateR <- function(browse_news, install_R, copy_packages, keep_old_packages,  u
    
    # if we got this far, the user wants to install the latest version of R (and his current version is old)
    cat("Installing the newest version of R,\n please wait for the installer file to be download and executed.\n Be sure to click 'next' as needed...\n")
-   did_R_install <- install.R(to_checkMD5sums = to_checkMD5sums, keep_install_file = keep_install_file, download_dir = download_dir) 
+   did_R_install <- install.R(to_checkMD5sums = to_checkMD5sums, keep_install_file = keep_install_file, download_dir = download_dir, silent = silent) 
    if(!did_R_install) return(FALSE) 
    new_R_path <- get.installed.R.folders()[1]
 
